exports.__esModule = true;
exports.default = void 0;

var _symbol = _interopRequireDefault(require("../core-js/symbol"));

var _instanceof2 = _interopRequireDefault(require("../helpers/instanceof"));

var _promise = _interopRequireDefault(require("../core-js/promise"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function AwaitValue(value) {
  this.value = value;
}

function AsyncGenerator(gen) {
  var front, back;

  function send(key, arg) {
    return new _promise.default(function (resolve, reject) {
      var request = {
        key: key,
        arg: arg,
        resolve: resolve,
        reject: reject,
        next: null
      };

      if (back) {
        back = back.next = request;
      } else {
        front = back = request;
        resume(key, arg);
      }
    });
  }

  function resume(key, arg) {
    try {
      var result = gen[key](arg);
      var value = result.value;

      if ((0, _instanceof2.default)(value, AwaitValue)) {
        _promise.default.resolve(value.value).then(function (arg) {
          resume("next", arg);
        }, function (arg) {
          resume("throw", arg);
        });
      } else {
        settle(result.done ? "return" : "normal", result.value);
      }
    } catch (err) {
      settle("throw", err);
    }
  }

  function settle(type, value) {
    switch (type) {
      case "return":
        front.resolve({
          value: value,
          done: true
        });
        break;

      case "throw":
        front.reject(value);
        break;

      default:
        front.resolve({
          value: value,
          done: false
        });
        break;
    }

    front = front.next;

    if (front) {
      resume(front.key, front.arg);
    } else {
      back = null;
    }
  }

  this._invoke = send;

  if (typeof gen.return !== "function") {
    this.return = undefined;
  }
}

if (typeof _symbol.default === "function" && _symbol.default.asyncIterator) {
  AsyncGenerator.prototype[_symbol.default.asyncIterator] = function () {
    return this;
  };
}

AsyncGenerator.prototype.next = function (arg) {
  return this._invoke("next", arg);
};

AsyncGenerator.prototype.throw = function (arg) {
  return this._invoke("throw", arg);
};

AsyncGenerator.prototype.return = function (arg) {
  return this._invoke("return", arg);
};

var _default = {
  wrap: function wrap(fn) {
    return function () {
      return new AsyncGenerator(fn.apply(this, arguments));
    };
  },
  await: function _await(value) {
    return new AwaitValue(value);
  }
};
exports.default = _default;